@@ -6,20 +6,25 @@ import os
 import tempfile
 
 import lore
-from lore.features.base import Base
+from lore.features.base import BaseFeatureExporter
 from lore.io import upload
 
 
-class S3(Base):
+class S3FeatureExporter(BaseFeatureExporter):
     __metaclass__ = ABCMeta
 
     @abstractmethod
     def serialization(self):
         pass
 
+    @property
+    def _data(self):
+        df = self.get_data()
+        return df
+
     def publish(self, compression='gzip'):
         temp_file, temp_path = tempfile.mkstemp(dir=lore.env.DATA_DIR)
-        data = self.get_data()
+        data = self._data
 
         if self.serialization() == 'csv':
             data.to_csv(temp_path, index=False, compression=compression)
@@ -36,7 +41,7 @@ class S3(Base):
         os.remove(temp_path)
 
     def data_path(self):
-        return "{}/{}/data.{}".format(self.version, self.name(), self.serialization())
+        return "{}/{}/data.{}".format(self.version, self.name, self.serialization())
 
     def metadata_path(self):
-        return "{}/{}/metadata.json".format(self.version, self.name())
+        return "{}/{}/metadata.json".format(self.version, self.name)
