@@ -5,12 +5,16 @@ import re
 import shutil
 import tarfile
 import tempfile
-
+try:
+    from urllib.parse import urlparse
+except ImportError:
+    from urlparse import urlparse
 
 import lore
 from lore.env import require, configparser
 from lore.util import timer
 from lore.io.connection import Connection
+from lore.io.multi_connection_proxy import MultiConnectionProxy
 
 
 logger = logging.getLogger(__name__)
@@ -32,14 +36,21 @@ if config:
         if options.get('url') == '$DATABASE_URL':
             logger.error('$DATABASE_URL is not set, but is used in config/database.cfg. Skipping connection.')
         else:
-            vars()[section.lower()] = Connection(name=section.lower(), **options)
+            if 'urls' in options:
+                vars()[section.lower()] = MultiConnectionProxy(name=section.lower(), **options)
+            else:
+                vars()[section.lower()] = Connection(name=section.lower(), **options)
+
+
+if 'metadata' not in vars():
+    vars()['metadata'] = Connection('sqlite:///%s/metadata.sqlite' % lore.env.DATA_DIR)
 
 redis_config = lore.env.REDIS_CONFIG
 if redis_config:
     require(lore.dependencies.REDIS)
     import redis
 
-    for section in config.sections():
+    for section in redis_config.sections():
         vars()[section.lower()] = redis.StrictRedis(host=redis_config.get(section, 'url'),
                                                     port=redis_config.get(section, 'port'))
 
@@ -65,14 +76,22 @@ if lore.env.AWS_CONFIG:
 
 
 def download(remote_url, local_path=None, cache=True, extract=False):
+    _bucket = bucket
     if re.match(r'^https?://', remote_url):
         protocol = 'http'
+    elif re.match(r'^s3?://', remote_url):
+        require(lore.dependencies.S3)
+        import boto3
+        from botocore.exceptions import ClientError
+        protocol = 's3'
+        url_parts = urlparse(remote_url)
+        remote_url = url_parts.path[1:]
+        _bucket = boto3.resource('s3').Bucket(url_parts.netloc)
     else:
-        if s3 is None:
+        if s3 is None or bucket is None:
             raise NotImplementedError("Cannot download from s3 without config/aws.cfg")
         protocol = 's3'
         remote_url = prefix_remote_root(remote_url)
-
     if cache:
         if local_path is None:
             if protocol == 'http':
@@ -90,11 +109,12 @@ def download(remote_url, local_path=None, cache=True, extract=False):
 
     with timer('DOWNLOAD: %s' % remote_url):
         temp_file, temp_path = tempfile.mkstemp(dir=lore.env.WORK_DIR)
+        os.close(temp_file)
         try:
             if protocol == 'http':
                 lore.env.retrieve_url(remote_url, temp_path)
             else:
-                bucket.download_file(remote_url, temp_path)
+                _bucket.download_file(remote_url, temp_path)
         except ClientError as e:
             logger.error("Error downloading file: %s" % e)
             raise
