@@ -11,6 +11,7 @@ import re
 import sys
 import tempfile
 import threading
+import uuid
 
 from datetime import datetime
 
@@ -78,7 +79,7 @@ class Connection(object):
     UNLOAD_PREFIX = os.path.join(lore.env.NAME, 'unloads')
     IAM_ROLE = os.environ.get('IAM_ROLE', None)
 
-    def __init__(self, url, name='connection', **kwargs):
+    def __init__(self, url, name='connection', watermark=True, **kwargs):
         if not sqlalchemy:
             raise lore.env.ModuleNotFoundError('No module named sqlalchemy. Please add it to requirements.txt.')
 
@@ -101,30 +102,33 @@ class Connection(object):
             del kwargs['__name__']
         if 'echo' not in kwargs:
             kwargs['echo'] = False
+        logger.info("Creating engine: %s %s" % (url, kwargs))
         self._engine = sqlalchemy.create_engine(url, **kwargs).execution_options(autocommit=True)
         self._metadata = None
         self.name = name
+        self.url = url
         self._transactions = []
         self.__thread_local = threading.local()
 
         @event.listens_for(self._engine, "before_cursor_execute", retval=True)
         def comment_sql_calls(conn, cursor, statement, parameters, context, executemany):
             conn.info.setdefault('query_start_time', []).append(datetime.now())
-            stack = inspect.stack()[1:-1]
-            if sys.version_info.major == 3:
-                stack = [(x.filename, x.lineno, x.function) for x in stack]
-            else:
-                stack = [(x[1], x[2], x[3]) for x in stack]
+            if watermark:
+                stack = inspect.stack()[1:-1]
+                if sys.version_info.major == 3:
+                    stack = [(x.filename, x.lineno, x.function) for x in stack]
+                else:
+                    stack = [(x[1], x[2], x[3]) for x in stack]
 
-            paths = [x[0] for x in stack]
-            origin = next((x for x in paths if x.startswith(lore.env.ROOT)), None)
-            if origin is None:
-                origin = next((x for x in paths if 'sqlalchemy' not in x), None)
-            if origin is None:
-                origin = paths[0]
-            caller = next(x for x in stack if x[0] == origin)
+                paths = [x[0] for x in stack]
+                origin = next((x for x in paths if x.startswith(lore.env.ROOT)), None)
+                if origin is None:
+                    origin = next((x for x in paths if 'sqlalchemy' not in x), None)
+                if origin is None:
+                    origin = paths[0]
+                caller = next(x for x in stack if x[0] == origin)
 
-            statement = "/* %s | %s:%d in %s */\n" % (lore.env.APP, caller[0], caller[1], caller[2]) + statement
+                statement = "/* %s | %s:%d in %s */\n" % (lore.env.APP, caller[0], caller[1], caller[2]) + statement
             return statement, parameters
 
         @event.listens_for(self._engine, "after_cursor_execute")
@@ -185,15 +189,20 @@ class Connection(object):
                     tmp = tempfile.NamedTemporaryFile(delete=False, suffix='.csv.gz')
                     tmp.close()
                     batch.to_csv(tmp.name, index=False, header=False, sep='|', na_rep='\\N', quoting=csv.QUOTE_NONE, compression='gzip')
-                    self._connection.connection.cursor().execute('PUT file://%(path)s @~/staged;' % {'path': tmp.name})
+                    suffix = str(uuid.uuid4())
+                    stage_name = 'staged_' + suffix
+                    self.execute('REMOVE @~/%(stage_name)s' % {'stage_name': stage_name})
+                    self._connection.connection.cursor().execute('PUT file://%(path)s @~/%(stage_name)s;' % {'path': tmp.name, 'stage_name': stage_name})
                     self._connection.connection.cursor().execute(
                         'COPY INTO %(table)s '
-                        'FROM @~/staged '
+                        'FROM @~/%(stage_name)s '
                         'FILE_FORMAT = (TYPE = CSV FIELD_DELIMITER = \'|\' SKIP_HEADER = 0 COMPRESSION = GZIP) '
                         'PURGE = TRUE' % {
-                            'table': table
+                            'table': table,
+                            'stage_name': stage_name
                         }
                     )
+                    self.execute('REMOVE @~/%(stage_name)s' % {'stage_name': stage_name})
                 finally:
                     os.remove(tmp.name)
 
@@ -329,7 +338,7 @@ class Connection(object):
             return result
 
     @query_cached
-    def load_dataframe(self, key, columns):
+    def load_dataframe(self, key, columns, log_verbose=False,):
         with timer('load_dataframe'):
             frames = []
             for entry in lore.io.bucket.objects.filter(
@@ -349,20 +358,21 @@ class Connection(object):
 
             result = pandas.concat(frames)
             result.columns = columns
-            buffer = io.StringIO()
-            result.info(buf=buffer, memory_usage='deep')
-            logger.info(buffer.getvalue())
-            logger.info(result.head())
+            if log_verbose is True:
+                buffer = io.StringIO()
+                result.info(buf=buffer, memory_usage='deep')
+                logger.info(buffer.getvalue())
+                logger.info(result.head())
             return result
 
-    def dataframe(self, sql=None, extract=None, filename=None, **kwargs):
+    def dataframe(self, sql=None, extract=None, filename=None, log_verbose=False, **kwargs):
         cache = kwargs.pop('cache', False)
         chunksize = kwargs.pop('chunksize', None)
         if chunksize and cache:
             raise ValueError('Chunking is incompatible with caching. Choose to pass either "cache" or "chunksize".')
         sql = self.__prepare(sql=sql, extract=extract, filename=filename, **kwargs)
         dataframe = self._dataframe(sql, kwargs, cache=cache, chunksize=chunksize)
-        if chunksize is None:
+        if log_verbose is True and chunksize is None:
             buffer = io.StringIO()
             dataframe.info(buf=buffer, memory_usage='deep')
             logger.info(buffer.getvalue())
