@@ -15,8 +15,8 @@ import threading
 import traceback
 from contextlib import contextmanager
 from datetime import datetime
-import inspect
 from lore import ansi, env
+from urllib.parse import urlparse
 
 
 class SecretFilter(logging.Filter):
@@ -111,6 +111,7 @@ class PrintInterceptor(object):
         self.errors = stream.errors
         self.encoding = stream.encoding
         self.level = level
+        self.isatty = stream.isatty
 
     def write(self, value):
         self.stream.write(value)
@@ -222,7 +223,7 @@ def parametrized(decorator):
 
 
 @parametrized
-def timed(func, level):
+def timed(func, level=logging.INFO):
     @functools.wraps(func)
     def wrapper(*args, **kwargs):
         with timer('.'.join([func.__module__, func.__name__]), level=level):
@@ -416,3 +417,76 @@ if env.launched():
         if _librato_timer:
             _librato_timer.cancel()
             _librato_timer = None
+
+
+def convert_df_columns_to_json(df, columns):
+    import pandas
+    import json
+
+    series = pandas.Series(df[columns].to_dict(orient='records'), index=df.index)
+    series = series.apply(lambda x: json.dumps(x, default=str))
+    return series
+
+
+def sql_alchemy_object_as_dict(obj):
+    from sqlalchemy import inspect
+    return {c.key: getattr(obj, c.key)
+            for c in inspect(obj).mapper.column_attrs}
+
+
+# From memoized_property package, copied source so as to prevent adding dependencies
+def memoized_property(fget):
+    """
+    Return a property attribute for new-style classes that only calls its getter on the first
+    access. The result is stored and on subsequent accesses is returned, preventing the need to
+    call the getter any more.
+    Example::
+        >>> class C(object):
+        ...     load_name_count = 0
+        ...     @memoized_property
+        ...     def name(self):
+        ...         "name's docstring"
+        ...         self.load_name_count += 1
+        ...         return "the name"
+        >>> c = C()
+        >>> c.load_name_count
+        0
+        >>> c.name
+        "the name"
+        >>> c.load_name_count
+        1
+        >>> c.name
+        "the name"
+        >>> c.load_name_count
+        1
+    """
+    attr_name = '_{0}'.format(fget.__name__)
+
+    @functools.wraps(fget)
+    def fget_memoized(self):
+        if not hasattr(self, attr_name):
+            setattr(self, attr_name, fget(self))
+        return getattr(self, attr_name)
+
+    return property(fget_memoized)
+
+
+# https://thesmithfam.org/blog/2012/10/25/temporarily-suppress-console-output-in-python/
+@contextmanager
+def suppress_stdout():
+    with open(os.devnull, "w") as devnull:
+        old_stdout = sys.stdout
+        sys.stdout = devnull
+        try:
+            yield
+        finally:
+            sys.stdout = old_stdout
+
+
+# only return '<hostname>:<port>/<path>' of given url - drop username, password, etc.
+def scrub_url(db_url):
+    parsed = urlparse(db_url)
+    host = getattr(parsed, 'hostname', None) or '_'
+    port = getattr(parsed, 'port', None) or '_'
+    path = getattr(parsed, 'path', None) or '_'
+    return "{}:{}{}".format(host, port, path)
